<?php

namespace TheBugs\Email;

class SwiftMailer implements MailerInterface
{
    protected $emailConfiguration;
    
    protected $transport;
    
    protected /*stringable*/ $from;
    
    protected /*stringable|array*/ $to;
    
    protected /*stringable|array*/ $cc;
    
    protected /*stringable|array*/ $bcc;

    protected array $attachments = [];
    
    public function __construct(EmailConfiguration $configuration) {
        $this->emailConfiguration = $configuration;
        
        $this->transport = ( new \Swift_SmtpTransport($this->emailConfiguration->smtpHost, $this->emailConfiguration->smtpPort) )
            ->setUsername($this->emailConfiguration->smtpUsername)
            ->setPassword($this->emailConfiguration->smtpPassword);
    }
    
    public function send(string $subject, string $message, bool $html = true) : bool
    {
        $swiftObj = ( new \Swift_Message($subject) )
            ->setFrom( $this->from )
            ->setTo( $this->to )
            ->setCC($this->cc ?? [])
            ->setBCC($this->bcc ?? [])
            ->setBody( $message, $html ? 'text/html' : 'text/plain' );

        foreach ($this->attachments as $attachment) {
            $swiftObj->attach($attachment);
        }

        $this->attachments = [];

        return ( new \Swift_Mailer($this->transport) )->send($swiftObj);
    }

    public function attach(\Swift_Attachment $attachment) : self
    {
        $this->attachments[] = $attachment;

        return $this;
    }
    
    public function setFrom(/*stringable|array*/ $from) : MailerInterface
    {
        $this->from = $from;
        
        return $this;
    } 
    
    public function setTo(/*stringable|array*/ $to) : MailerInterface
    {
        $this->to = $to;
        
        return $this;
    }
    
    public function setCC(/*stringable|array*/ $cc) : MailerInterface
    {
        $this->cc = $cc;
        
        return $this;
    }
    
    public function setBCC(/*stringable|array*/ $bcc) : MailerInterface
    {
        $this->bcc = $bcc;
        
        return $this;
    }
}